

#include <windows.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "BladeMP3EncDLL.h"

BEINITSTREAM		beInitStream=NULL;
BEENCODECHUNK		beEncodeChunk=NULL;
BEDEINITSTREAM		beDeinitStream=NULL;
BECLOSESTREAM		beCloseStream=NULL;
BEVERSION			beVersion=NULL;
BEWRITEVBRHEADER	beWriteVBRHeader=NULL;
BEWRITEINFOTAG		beWriteInfoTag=NULL;


// Main program
BOOL Wav2Mp3(char *strFileIn, char *strFileOut)
{	
	HINSTANCE	hDLL			=NULL;
	FILE*		pFileIn			=NULL;
	FILE*		pFileOut		=NULL;
	BE_CONFIG	beConfig		={0,};

	DWORD		dwSamples		=0;
	DWORD		dwMP3Buffer		=0;
	HBE_STREAM	hbeStream		=0;
	BE_ERR		err				=0;

	PBYTE		pMP3Buffer		=NULL;
	PSHORT		pWAVBuffer		=NULL;

	BOOL		ret_val			=FALSE;
	BOOL		temp_err		=FALSE;

	do {
		// Load lame_enc.dll library (Make sure though that you set the 
		// project/settings/debug Working Directory correctly, otherwhise the DLL can't be loaded

		if (!(hDLL = LoadLibrary("lame_enc.dll")))
			break;

		// Get Interface functions from the DLL
		beInitStream	= (BEINITSTREAM) GetProcAddress(hDLL, TEXT_BEINITSTREAM);
		beEncodeChunk	= (BEENCODECHUNK) GetProcAddress(hDLL, TEXT_BEENCODECHUNK);
		beDeinitStream	= (BEDEINITSTREAM) GetProcAddress(hDLL, TEXT_BEDEINITSTREAM);
		beCloseStream	= (BECLOSESTREAM) GetProcAddress(hDLL, TEXT_BECLOSESTREAM);
		beVersion		= (BEVERSION) GetProcAddress(hDLL, TEXT_BEVERSION);
		beWriteVBRHeader= (BEWRITEVBRHEADER) GetProcAddress(hDLL,TEXT_BEWRITEVBRHEADER);
		beWriteInfoTag  = (BEWRITEINFOTAG) GetProcAddress(hDLL,TEXT_BEWRITEINFOTAG);

		// Check if all interfaces are present
		if(!beInitStream || !beEncodeChunk || !beDeinitStream || !beCloseStream || !beVersion || !beWriteVBRHeader)
			break;

		// Try to open the WAV file, be sure to open it as a binary file!	
		if ( !(pFileIn = fopen( strFileIn, "rb" )))
			break;

		// Open MP3 file
		if ( !(pFileOut= fopen(strFileOut,"wb+"))) 
			break;

		memset(&beConfig,0,sizeof(beConfig));					// clear all fields

		fseek(pFileIn,24,SEEK_SET);
		fread(&beConfig.format.LHV1.dwSampleRate, 4, 4, pFileIn);

		// use the LAME config structure
		beConfig.dwConfig = BE_CONFIG_LAME;

		// this are the default settings for testcase.wav
		beConfig.format.LHV1.dwStructVersion	= 1;
		beConfig.format.LHV1.dwStructSize		= sizeof(beConfig);		
		//beConfig.format.LHV1.dwSampleRate		= 48000;				// INPUT FREQUENCY
		beConfig.format.LHV1.dwReSampleRate		= 0;					// DON"T RESAMPLE
		beConfig.format.LHV1.nMode				= BE_MP3_MODE_JSTEREO;	// OUTPUT IN STREO
		beConfig.format.LHV1.dwBitrate			= 64;					// MINIMUM BIT RATE
		beConfig.format.LHV1.nPreset			= LQP_R3MIX;		// QUALITY PRESET SETTING
		beConfig.format.LHV1.dwMpegVersion		= MPEG1;				// MPEG VERSION (I or II)
		beConfig.format.LHV1.dwPsyModel			= 0;					// USE DEFAULT PSYCHOACOUSTIC MODEL 
		beConfig.format.LHV1.dwEmphasis			= 0;					// NO EMPHASIS TURNED ON
		beConfig.format.LHV1.bOriginal			= TRUE;					// SET ORIGINAL FLAG
		beConfig.format.LHV1.bWriteVBRHeader	= TRUE;					// Write INFO tag

	//	beConfig.format.LHV1.dwMaxBitrate		= 320;					// MAXIMUM BIT RATE
	//	beConfig.format.LHV1.bCRC				= TRUE;					// INSERT CRC
	//	beConfig.format.LHV1.bCopyright			= TRUE;					// SET COPYRIGHT FLAG	
	//	beConfig.format.LHV1.bPrivate			= TRUE;					// SET PRIVATE FLAG
	//	beConfig.format.LHV1.bWriteVBRHeader	= TRUE;					// YES, WRITE THE XING VBR HEADER
	//	beConfig.format.LHV1.bEnableVBR			= TRUE;					// USE VBR
	//	beConfig.format.LHV1.nVBRQuality		= 5;					// SET VBR QUALITY
		beConfig.format.LHV1.bNoRes				= TRUE;					// No Bit resorvoir

	//  Preset Test
	//	beConfig.format.LHV1.nPreset			= LQP_PHONE;

		// Init the MP3 Stream
		err = beInitStream(&beConfig, &dwSamples, &dwMP3Buffer, &hbeStream);

		// Check result
		if(err != BE_ERR_SUCCESSFUL)
			break;

		// Allocate MP3 buffer
		pMP3Buffer = new BYTE[dwMP3Buffer];

		// Allocate WAV buffer
		pWAVBuffer = new SHORT[dwSamples];

		// Check if Buffer are allocated properly
		if(!pMP3Buffer || !pWAVBuffer)
			break;

		DWORD dwRead=0;
		DWORD dwWrite=0;
		DWORD dwDone=0;
		DWORD dwFileSize=0;

		// Seek to end of file
		fseek(pFileIn,0,SEEK_END);

		// Get the file size
		dwFileSize=ftell(pFileIn);

		// Seek back to start of WAV file,
		// but skip the first 44 bytes, since that's the WAV header
		fseek(pFileIn,44,SEEK_SET);

		// Convert All PCM samples

		while ( (dwRead=fread(pWAVBuffer,sizeof(SHORT),dwSamples,pFileIn)) >0 ) {
			// Encode samples
			err = beEncodeChunk(hbeStream, dwRead, pWAVBuffer, pMP3Buffer, &dwWrite);

			// Check result
			if(err != BE_ERR_SUCCESSFUL) {
				temp_err = TRUE;
				break;
			}
			
			// write dwWrite bytes that are returned in tehe pMP3Buffer to disk
			if(fwrite(pMP3Buffer,1,dwWrite,pFileOut) != dwWrite) {
				temp_err = TRUE;
				break;
			}

			dwDone += dwRead*sizeof(SHORT);
		}
		if (temp_err)
			break;

		// Deinit the stream
		err = beDeinitStream(hbeStream, pMP3Buffer, &dwWrite);

		// Check result
		if(err != BE_ERR_SUCCESSFUL)
			break;

		// Are there any bytes returned from the DeInit call?
		// If so, write them to disk
		if( dwWrite )
			if( fwrite( pMP3Buffer, 1, dwWrite, pFileOut ) != dwWrite )
				break;

		if ( beWriteInfoTag )
			// Write the INFO Tag
			beWriteInfoTag( hbeStream, strFileOut );
		else
			beWriteVBRHeader( strFileOut );

		ret_val = TRUE;
	} while(0);

	if (hbeStream) beCloseStream(hbeStream);
	if (hDLL) FreeLibrary(hDLL);
	if (pFileIn) fclose(pFileIn);
	if (pFileOut) fclose(pFileOut);
	if (pMP3Buffer) delete [] pMP3Buffer;
	if (pWAVBuffer) delete [] pWAVBuffer;
	if (!ret_val)
		DeleteFile(strFileOut);

	return ret_val;
}

